
> module Test where

> import System
> import Basics
> import Ratio
> import Scores
> import PerformanceEvaluation
> import ComposersWorkbench
> import IO 
> import Miscellany
> import SaxophoneFingerings
> import Performer
> import EtudeGeneration
> import ScoresExample
> import EtudeGenerationExample

************
* Utilities
************

write some text to a file

> writeTextFile url contents 
>   = do toHandle <- openFile url  WriteMode 
>        hPutStr toHandle contents
>        hClose toHandle
>        putStr "Done Writing File.\n"

> lilyPond input output = do putStr $ "Running command: " ++ cmd ++ "\n"
>                            system cmd
>   where cmd = "/Applications/LilyPond/lilypond.sh --pdf --output=" ++ output ++ " " ++ input

Use OS X to open the file in the default viewer

> open url = system $ "open " ++ url

Generate a file, compile it in Lilypond, and open the PDF in the viewer

> genCompileAndShow :: String -> String -> LilyPondScore -> IO ()
> genCompileAndShow name parentDir s 
>  = do putStr $ "Writing lily file: " ++ lilyFile ++ "\n"
>       writeTextFile lilyFile (renderLPScore s)
>       lilyPond lilyFile urlStart
>       -- @todo mv cmd is required since lilyPnd does the worng thing?
>       system $ "mv " ++ name ++ ".pdf " ++ urlPDF
>       open urlPDF
>       putStr "Done Processing\n"
>    where urlStart = parentDir ++ "/" ++ name
>          lilyFile = urlStart ++ ".ly"
>          urlPDF   = urlStart ++ ".pdf" 

> showPWD = system "pwd"

*************************
* Scores
*************************

Mary Had a Little Lamb -- the musician's "Hello World"

> maryScore :: Score
> maryScore = Score "Mary Had a Little Lamb" (4%4) 138 (Key C Major) maryMusic

> maryMusic :: Music
> maryMusic = sequencePhrases maryPhrases

> maryPhrases :: [Music]
> maryPhrases = [phrase1, (phrase2 :+: note2), phrase1, (phrase2 :+: qnr)]
>  where note1 = e oct qn []
>        note2 = d oct qn []
>        note3 = c oct qn []
>        oct   = 4
>        n1232 = note1 :+: note2 :+: note3 :+: note2 
>        n11   = note1 :+: note1
>        n22   = note2 :+: note2
>        phrase1 = n1232 :+: n11 :+: (e oct hn []) :+: n22 :+: (d oct hn []) :+: note1 :+: (g oct qn []) :+: (g oct hn [])
>        phrase2 =  phrase2' :+: (c oct dhn [])
>        phrase2' = n1232 :+: n11 :+: n11 :+: n22 :+: note1 :+: note2

Test of tieing and dotting

> durationTest :: Score
> durationTest = Score "Assorted Sixteenth Notes" (4%4) 60 (Key C Major) m
>  where m       = makeMelody (12*5) pitches rhythm
>        pitches = (take 15 (repeat 0))
>        rhythm  = (foldr1 (:&:) dtNotes)

> dtNotes = [(NoteR (x%16)) :&: (RestR ((16-x)%16)) | x <- [1..15]]

************************************
* Etude Generation: Round Trip Test
************************************

Start with the default performer.  Add score1 to their on-deck list.  Score1 has several phrases, which share some notes in common.  Then add performance evaluation for score1, where some phrases were successful and others weren't.  Then select a phrase or phrases of score1 (the unsuccessful ones) and use them to generate a new etude.  Then print out that etude.  (what does it look like -- is it reasonable?)  

so we're going to use umtandv with first two phrases succ and last phrase unsucc

> writeRTTEtude2 = genCompileAndShow "roundTripTest2" outputDir (score2LPScore rttEtude2) 

The rest of this section is the result of some debugging I did to determine what's wrong with the etude generation.  I don't wat to clean it up until I have fixed the bug, so please skip ahead to the next section.

@todo this one fails cuz its too long...

> writeRTTEtude = genCompileAndShow "roundTripTest" outputDir (score2LPScore rttEtude) 

> rttEtude :: Score
> rttEtude = generateEtude tm ts title
>   where title = "Round Trip Test: Etude Based on UM T and V"
>         ts = rtteTrans
>         tm = rtteTM

> rtteTM :: TechniqueMap
> rtteTM = updateWithEvals [umTandV] pes tm0 -- technique map for the performer, updated with performance evals
>   where tm0 = rtteTM0
>         pes :: [PerformanceEvaluation]
>         pes = [CanPlay (PhraseNum 0 t), CanPlay (PhraseNum 1 t), CannotPlay (PhraseNum 2 t) TrickyFingering]
>         (Score t _ _ _ _) = umTandV -- the score that the performer was trained on

> rtteTM0 = tm0 
>   where (Performer _ (Abilities tm0)) = defPerformer -- the performer, in initial state

Debugging...

The problem is with fingeringCombs or fingeringTempos when called by updateBy???... try to rep. there...

@todo the following is blowing the stack .. debug

> rtteTest3 :: TechniqueMap
> rtteTest3 = updateWithEval [umTandV] (CannotPlay (PhraseNum 2 t) TrickyFingering) rtteTest2
>   where (Score t _ _ _ _) = umTandV -- the score that the performer was trained on

< rtteTest41 = fingeringCombinations rtteTestM -- this definately reproduces the bug...

> rtteTest411 = findFingeringSequence rtteTestM -- this is the bug: this function is too inefficient!!!

> rtteTest41 = zipSeq (map fingeringNumber rtteTest411)  -- this definately reproduces the bug...

> rtteTest4 = zip rtteTest41 rtteTest42 -- this definately reproduces the bug...

> rtteTest42 = fingeringTempos rtteTestM bpm bpMeas
>   where bpMeas = timeSigToBPMeas timeS
>         (Score t timeS bpm _ _) = umTandV

> rtteTestM = phraseNum 2 umTandV

> rtteTest5 = fTest rtteTest4 rtteTM0 

This is the function f from updateBy

> fTest :: [(SaxFCombination, FingeringTempo)] -> TechniqueMap -> TechniqueMap
> fTest []     t' = t'
> fTest (x:s') t' = fTest s' (updateTFail x t')

> updateBy :: ((SaxFCombination, FingeringTempo) -> TechniqueMap -> TechniqueMap) -> Music -> Integer -> Integer -> TechniqueMap -> TechniqueMap
> updateBy g m bpm bpMeas t = f (zip s ft) t 
>   where s :: [SaxFCombination]
>         s  = fingeringCombinations m
>         ft = fingeringTempos m bpm bpMeas
>         f :: [(SaxFCombination, FingeringTempo)] -> TechniqueMap -> TechniqueMap
>         f []     t' = t'
>         f (x:s') t' = f s' (g x t')

End debugging...

@todo these fnss are fine...

> rtteTest2 :: TechniqueMap
> rtteTest2 = updateWithEval [umTandV] (CanPlay (PhraseNum 1 t)) rtteTest1
>  where (Score t _ _ _ _) = umTandV -- the score that the performer was trained on

> rtteTest1 :: TechniqueMap
> rtteTest1 = updateWithEval [umTandV] (CanPlay (PhraseNum 0 t)) rtteTM0
>  where (Score t _ _ _ _) = umTandV -- the score that the performer was trained on

> rtteTrans :: [FingeringTrans] -- the set of transitions from the phrase that we'd like to use
> rtteTrans = transitionsToTest rtteTM (phraseNum 2 umTandV) bpm (timeSigToBPMeas timeS)
>   where (Score _ timeS bpm _ _) = umTandV -- the score that the performer was trained on

@todo move the following back to PerformanceEvaluation.lhs

************************** 
* Typesetting Tests 
**************************

> outputDir = "/Users/matt/Documents/notes/cs-masters/research/haskell-music/output"
> sampleFile = outputDir ++ "/sample.2.ly"
> samplePDF = outputDir ++ "/" ++ "sample.2.lilyPond"
> samplePDFFile = samplePDF ++ ".pdf"

> sampleScore :: LilyPondScore
> sampleScore = [(LPComment "Sample ly file generated by Haskell program by Matt Munz"),
> 	         (LPHeader "Hello World: A scale in LilyPond"),
>                Relative,
>                (LPBlock ([(LPNote a b) | (a, b) <- notes] ++ [FinalBar])), 
>                (LPVersion "2.6.0")]
>  where notes = [("c", "") , ("d", "16"), ("e", "8"), ("f", "16"), ("b", "8"), ("b", "16"), ("g", "32"), ("a", "32"), ("c", "4"), ("f", "1"), ("a", "1")]

> writeSampleFile :: IO ()
> writeSampleFile = genCompileAndShow "sample.4" outputDir sampleScore

> writeMaryFile = genCompileAndShow "mary" outputDir (score2LPScore maryScore)

> genUMTV = genCompileAndShow "universalMethodTandV" outputDir (score2LPScore umTandV)

> genDurT = genCompileAndShow "durationTest" outputDir (score2LPScore durationTest)

> genSimpleEtude = genCompileAndShow "simpleEtude" outputDir (score2LPScore simpleEtude)

> genMels = genCompileAndShow "sampleMels" outputDir (score2LPScore melsScore)
>   where melsScore = Score "Sample Saxophone Melodies, transposed for Eb Alto" (4%4) 100 (Key C Major) (transposeM 9 mel1)

> openSampleFile = open samplePDFFile

> compileSampleFile = lilyPond sampleFile samplePDF

******************************** 
* Performance Evaluation Tests 
********************************

> mattsPerformances :: [PerformanceEvaluation]
> mattsPerformances = mp ++ up 
>  where m  = case maryScore of
>               Score t _ _ _ _ -> t
>        u  = case umTandV of
>               Score t _ _ _ _ -> t
>        mp = [CanPlay (PhraseNum n m) | n <- [0..3]]
>        up = [CannotPlay (PhraseNum n u) TrickyFingering | n <- [0..2]]

****************************** 
* Difficulty Estimation tests
******************************

> diffTest1 = estimateDifficulty defPerformer (maryPhrases !! 0) 138 4

> diffTest2 = estimateDifficulty dp2 (maryPhrases !! 0) 138 4

> diffTest3 = estimateDifficulty dp2 (maryPhrases !! 0) 165 4

> diffTestX n = estimateDifficulty dp2 (maryPhrases !! 0) n 4

> dp2 = f defPerformer
>   where f (Performer _ (Abilities t)) = (Performer n (Abilities (updateRecords t s)))
>         n = "Sample Performer with some skills..." 
>         s = [(22,18),(18,13),(22,22),(18,18),(22,26),(26,26)]

> updateRecords :: TechniqueMap -> [SaxFCombination] -> TechniqueMap
> updateRecords t s = f' s
>  where  f' []     = t -- @todo refactor this out to a updateRecords fn in PerformanceEvaluation... make it more general
>         f' (x:xs) = updateRecord (f' xs) x 40 8


*******************************
* Saxophone Fingering Tests
*******************************

Some melodies to test saxophone fingering selection

in key of Eb: g f# e f g f# f

> mel1 = makeMelody (12*4 - 2) pcs rhythm
>   where pcs    = [0, -1, -3, -2, 0, -1, -2]
>         rhythm = (foldr1 (:&:) (take 6 (repeat (NoteR (1%4))))) :&: NoteR (1%2)

chromatic scale from bottom of horn up to top

> mel2 = makeMelody (12*3+1) [0..31] (foldr1 (:&:) (take 32 (repeat (NoteR (1%4)))))

c major scale from bottom of horn to top

> mel3 = makeMelody (12*3+3) (concat (map (\x -> map (+(12*x)) [0,2,4,5,7,9,11]) [0,1])) (foldr1 (:&:) (take 14 (repeat (NoteR (1%4)))))

prints the "correct" fingering sequence for a given melody

> selFingeringTest :: Music -> IO ()
> selFingeringTest m = putStr ("Fingering Sequence for Melody: " ++ (show m) ++ "\n\n" ++ diagrams ++ "\n")
>  where fs :: [SaxophoneFingering]
>        fs = findFingeringSequence m
>        diagrams = concat (map (\x -> "\n" ++ (fingeringDiagram x)) fs)

prints all the fingering sequences for a given melody

> printAllFingerings :: Music -> IO ()
> printAllFingerings m = putStr ("All Fingering Sequences for Melody: \n" ++ (show m) ++ "\n\n" ++ diagrams ++ "\n")
>  where fss :: [[SaxophoneFingering]]
>        fss = findAllFingerings m
>        diagrams = concat (map (\s -> "\n" ++ (diagram s) ++ "\n-------------------------") fss)
>        diagram s = concat (map (\x -> "\n" ++ (fingeringDiagram x)) s) -- @todo refactor with above

> mel1Test = selFingeringTest mel1
> mel1ATest = printAllFingerings mel1
> mel2Test = selFingeringTest mel2
> mel3Test = selFingeringTest mel3

**********************
* Other Tests
**********************

> durConvertTest = x where
>  x :: Int
>  x = floor (fromIRatio y)
> --  x = (intToFractional (numerator y)) / (intToFractional (denominator y))
>  y :: Dur
>  y = 5%7
>  intToFractional :: (Fractional a) => Int -> a 
>  intToFractional i = fromInteger (toInteger i)

> sampleDB = makePDB "output/MattMunz.PerformerDB.txt" "Matt Munz"

